'use strict';
const Command = require('@foundation-cli/command');
const Package = require('@foundation-cli/package');
const {
  spinnerStart,
  sleep
} = require('@foundation-cli/utils');
const log = require('@foundation-cli/log');
const getProjectTemplate = require('./getProjectTemplate');
const fs = require('fs');
const path = require('path');
const userHome = require('user-home');
const fse = require('fs-extra');
const inquirer = require('inquirer');
const semver = require('semver');
const colors = require('colors');

const TYPE_PROJECT = 'project';
const TYPE_COMPONENT = 'component';

class InitCommand extends Command {
  constructor(argv) {
    super(argv);
  }
  init() {
    this.projectName = this._argv[0] || '';
    this.opts = this._argv[1];
    log.verbose(this.projectName, this.opts);
  }
  async exec() {
    try {
      const projectInfo = await this.prepare(); // 初始化项目的预备工作, 同时返回一个json格式的项目信息对象

      log.verbose('projectInfo', projectInfo);
      this.projectInfo = projectInfo;
      await this.downloadTemplate();
    } catch (e) {
      log.error(e.message);
    }
  }
  async downloadTemplate() {
    // 1、通过项目模版API获取项目模版信息
    // 1.1、通过egg.js 搭建一套后端系统
    // 1.2、通过npm存储项目模版
    // 1.3、将项目模版信息存储到mongodb数据库中
    // 1.4、通过egg.js获取mongodb中的数据并通过API返回
    /*await getProjectTemplate().then((res) => {
      console.log(res.data);
    })*/
    // 获取template信息
    const templateInfo = this.template.find(item => item.npmName === this.projectInfo.projectTemplate);
    const targetPath = path.resolve(userHome, '.foundation-cli', 'template');
    const storePath = path.resolve(userHome, '.foundation-cli', 'template', 'node_modules');
    const {
      npmName,
      version
    } = templateInfo;
    const templateNpm = new Package({
      targetPath,
      storePath,
      packageName: npmName,
      packageVersion: version
    });

    // 下载最新的模版到本地缓存
    if (!(await templateNpm.exists())) {
      const spinner = spinnerStart('正在下载模版');

      await sleep();
      try {
        await templateNpm.install();
        log.success(colors.green('下载模版成功'));
      } catch (e) {
        log.error(colors.red(e.message));
      } finally {
        spinner.stop(true); // 传入true的话，在spinner stop的时候会清除spinner的loading文案
      }
    } else {
      const spinner = spinnerStart('正在更新模版');

      await sleep();
      try {
        await templateNpm.update();
        log.success(colors.green('更新模版成功'));
      } catch (e) {
        log.error(colors.red(e.message));
      } finally {
        spinner.stop(true);
      }
    }
  }
  async prepare() {
    // 判断项目模版是否存在
    const template = await getProjectTemplate();

    if (!template || !template.length) {
      throw new Error('项目模版不存在');
    }
    this.template = template;
    // 第一步：判断当前目录是否为空
    const localPath = process.cwd(); // 获取执行命令的目录，也可以使用path.resolve('.')获取
    if ('/Users/feng.shen/Desktop/init_project' !== localPath) {
      log.error('创建项目的目录不对，有误删文件的风险');
      return;
    }
    const isEmpty = this.isEmptyDir(localPath); // 第一步判断目录是否为空
    const force = this.opts.force; // force参数的含义：即使当前目录存在文件，也一定会初始化项目
    let ifContinue = false;

    if (!isEmpty) {
      if (!force) { // 如果用户没有传入--force参数，就要进行询问
        const answer = await inquirer.prompt({
          type: 'confirm',
          name: 'ifContinue',
          default: false,
          message: '当前目录不为空，是否继续初始化项目？'
        });

        ifContinue = answer.ifContinue;

        if (!ifContinue) {
          return; // 用户选择不继续就直接return
        }
      }
      if (ifContinue || force) { // 在目录中进行项目初始化
        const { confirmComplete } = await inquirer.prompt({
          type: 'confirm',
          name: 'confirmComplete',
          default: false,
          message: '是否清空当前目录里的文件？'
        });

        if (confirmComplete) {
          fse.emptyDirSync(localPath);
        }

        return await this.getProjectInfo();
      }
    } else {
      return await this.getProjectInfo();
    }
  }
  async getProjectInfo() { // 获取项目初始化的基本信息
    let projectInfo = {};
    // 第一步：选择创建项目还是组件
    const {
      type
    } = await inquirer.prompt({
      type: 'list',
      name: 'type',
      message: '请选择初始化类型',
      default: TYPE_PROJECT,
      choices: [{
        name: '项目',
        value: TYPE_PROJECT
      }, {
        name: '组件',
        value: TYPE_COMPONENT
      }]
    });

    log.verbose('type', type);

    // 第二步：获取项目或者组件的基本信息
    if (type === TYPE_PROJECT) {
      const o = await inquirer.prompt([
        {
          type: 'input',
          name: 'projectName',
          message: '请输入项目名称',
          default: this.projectName,
          validate: function(v) {
            let done = this.async();
            setTimeout(() => {
              /**
               * 规则如下：
               * 1、首字符必须都为英文字符
               * 2、尾字符必须为英文或者数字，不能是特殊字符
               * 3、名称中包含的特殊字符仅允许'-_'
               */
              if (!/^[a-zA-Z]+([-][a-zA-Z][a-zA-Z0-9]*|[_][a-zA-Z][a-zA-Z0-9]*|[a-zA-Z0-9])*$/.test(v)) { // 校验不通过
                done('You need to provide a valid name'); // 输入不合法时的提示信息
                return;
              }
              done(null, true);
            }, 0);
          },
          filter: function(v) {
            return v;
          }
        },
        {
          type: 'input',
          name: 'projectVersion',
          message: '请输入项目版本号',
          default: '1.0.0',
          validate: function(v) {
            let done = this.async();
            setTimeout(() => {
              if (!semver.valid(v)) { // 校验不通过
                done('请输入合法的版本号'); // 输入不合法时的提示信息
                return;
              }
              done(null, true);
            }, 0);
          },
          filter: function(v) {
            if (!!semver.valid(v)) {
              return semver.valid(v); // 比如传入v1.0.0，会帮我们转化为格式化为1.0.0
            } else {
              return v;
            }
          }
        },
        {
          type: 'list',
          name: 'projectTemplate',
          message: '请选择项目模版',
          choices: this.createTemplateChoice(),
          validate: function(v) {
            let done = this.async();
            setTimeout(() => {
              if (!semver.valid(v)) { // 校验不通过
                done('请输入合法的版本号'); // 输入不合法时的提示信息
                return;
              }
              done(null, true);
            }, 0);
          },
          filter: function(v) {
            if (!!semver.valid(v)) {
              return semver.valid(v); // 比如传入v1.0.0，会帮我们转化为格式化为1.0.0
            } else {
              return v;
            }
          }
        }
      ]);
      projectInfo = {
        ...o
      };
    } else if (type === TYPE_COMPONENT) {

    }

    return projectInfo;
  }
  createTemplateChoice() {
    return this.template.map((item) => ({
      value: item.npmName,
      name: item.name
    }));
  }
  isEmptyDir(path) {
    let files = fs.readdirSync(path);

    files = files.filter(file => { // 过滤一些特定的文件
      return !file.startsWith('.') && ['node_modules'].indexOf('file') < 0
    });

    return !(files && files.length);
  }
}

function init(argv) {
  /*const cmdObj = arguments[1];
  console.log('init core', cmdObj.force, process.env.CLI_TARGET_PATH);*/

  return new InitCommand(argv);
}

module.exports.InitCommand = InitCommand;

module.exports = init;
